
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Gowin_EMPU_Template is
    port(
        sys_clk:    in std_logic;

        uart0_rxd: in std_logic;
        uart0_txd: out std_logic;

        --Embedded Hyperram Interface
        O_hpram_ck: out std_logic_vector(0 downto 0);
        O_hpram_ck_n: out std_logic_vector(0 downto 0);
        O_hpram_cs_n: out std_logic_vector(0 downto 0);
        O_hpram_reset_n: out std_logic_vector(0 downto 0);
        IO_hpram_dq: inout std_logic_vector(7 downto 0);
        IO_hpram_rwds: inout std_logic_vector(0 downto 0);

        --Gamboy Interface
        nWR     : in std_logic;
        nRD     : in std_logic;
        nCS     : in std_logic;
        A       : in std_logic_vector(14 downto 0);
        A15     : in std_logic;
        D       : inout std_logic_vector(7 downto 0);
        DIR     : out std_logic;
        nOE     : out std_logic;
        nRST    : out std_logic
        );
end Gowin_EMPU_Template;

architecture rtl of Gowin_EMPU_Template is

signal master_hclk: std_logic;
signal master_hrst: std_logic;
signal master_hsel: std_logic;
signal master_haddr: std_logic_vector(31 downto 0);
signal master_htrans: std_logic_vector(1 downto 0);
signal master_hwrite: std_logic;
signal master_hsize: std_logic_vector(2 downto 0);
signal master_hburst: std_logic_vector(2 downto 0);
signal master_hprot: std_logic_vector(3 downto 0);
signal master_memattr: std_logic_vector(1 downto 0);
signal master_exreq: std_logic;
signal master_hmaster: std_logic_vector(3 downto 0);
signal master_hwdata: std_logic_vector(31 downto 0);
signal master_hmastlock: std_logic;
signal master_hreadymux: std_logic;
signal master_hauser: std_logic;
signal master_hwuser: std_logic_vector(3 downto 0);
signal master_hrdata: std_logic_vector(31 downto 0);
signal master_hreadyout: std_logic;
signal master_hresp: std_logic;

signal mclk: std_logic ;             --mcu clock = 27MHz
signal hpram_memory_clk: std_logic;  --HyperRAM memory clock = 159MHz
signal hpram_base_clk: std_logic;    --HyperRAM base clock = 79.5MHz


component Gowin_PLLVR
    port (
        clkout: out std_logic;   --hpram_memory_clk 159MHz
        clkoutd: out std_logic;  --hpram_base_clk 79.5MHz
        clkin: in std_logic      
    );                           
end component;

--AHB HyperRAM instantiation
--HyperRAM Memory Interface
--Memory Clock = 100MHz
component Gowin_AHB_HyperRAM_Top
    port(
        --AHB
        AHB_HRDATA: out std_logic_vector(31 downto 0);
        AHB_HREADY: out std_logic;
        AHB_HRESP: out std_logic;
        AHB_HTRANS: in std_logic_vector(1 downto 0);
        AHB_HBURST: in std_logic_vector(2 downto 0);
        AHB_HPROT: in std_logic_vector(3 downto 0);
        AHB_HSIZE: in std_logic_vector(2 downto 0);
        AHB_HWRITE: in std_logic;
        AHB_HMASTLOCK: in std_logic;
        AHB_HMASTER: in std_logic_vector(3 downto 0);
        AHB_HADDR: in std_logic_vector(31 downto 0);
        AHB_HWDATA: in std_logic_vector(31 downto 0);
        AHB_HSEL: in std_logic;
        AHB_HCLK: in std_logic;
        AHB_HRESETn: in std_logic;

        --Gamboy Interface
        nWR     : in std_logic;
        nRD     : in std_logic;
        nCS     : in std_logic;
        A       : in std_logic_vector(14 downto 0);
        A15     : in std_logic;
        D       : inout std_logic_vector(7 downto 0);
        DIR     : out std_logic;
        nOE     : out std_logic;
        
        --HYPERRAM
        W_HPRAM_WRDATA: out std_logic_vector(31 downto 0);
        hpram_data_out: in std_logic_vector(31 downto 0);
        rd_valid: in std_logic;
        W_reg_address: out std_logic_vector(21 downto 0);
        W_cross_reg_cmd2: out std_logic;
        W_cmd_enable: out std_logic;
        wire_init: in std_logic;
        user_clk: in std_logic;
        data_mask: out std_logic_vector(3 downto 0);

        --GB
        GB_RESET: out std_logic;
        S_GB_nAHB_MODE: out std_logic
        );
end component;

component HyperRAM_Memory_Interface_Top
	port (
		clk: in std_logic;
		memory_clk: in std_logic;
		pll_lock: in std_logic;
		rst_n: in std_logic;
		O_hpram_ck: out std_logic_vector(0 downto 0);
		O_hpram_ck_n: out std_logic_vector(0 downto 0);
		IO_hpram_dq: inout std_logic_vector(7 downto 0);
		IO_hpram_rwds: inout std_logic_vector(0 downto 0);
		O_hpram_cs_n: out std_logic_vector(0 downto 0);
		O_hpram_reset_n: out std_logic_vector(0 downto 0);
		wr_data: in std_logic_vector(31 downto 0);
		rd_data: out std_logic_vector(31 downto 0);
		rd_data_valid: out std_logic;
		addr: in std_logic_vector(21 downto 0);
		cmd: in std_logic;
		cmd_en: in std_logic;
		init_calib: out std_logic;
		clk_out: out std_logic;
		data_mask: in std_logic_vector(3 downto 0)
	);
end component;


component Gowin_EMPU_Top
	port (
		sys_clk: in std_logic;
		uart0_rxd: in std_logic;
		uart0_txd: out std_logic;
		master_hclk: out std_logic;
		master_hrst: out std_logic;
		master_hsel: out std_logic;
		master_haddr: out std_logic_vector(31 downto 0);
		master_htrans: out std_logic_vector(1 downto 0);
		master_hwrite: out std_logic;
		master_hsize: out std_logic_vector(2 downto 0);
		master_hburst: out std_logic_vector(2 downto 0);
		master_hprot: out std_logic_vector(3 downto 0);
		master_memattr: out std_logic_vector(1 downto 0);
		master_exreq: out std_logic;
		master_hmaster: out std_logic_vector(3 downto 0);
		master_hwdata: out std_logic_vector(31 downto 0);
		master_hmastlock: out std_logic;
		master_hreadymux: out std_logic;
		master_hauser: out std_logic;
		master_hwuser: out std_logic_vector(3 downto 0);
		master_hrdata: in std_logic_vector(31 downto 0);
		master_hreadyout: in std_logic;
		master_hresp: in std_logic;
		master_exresp: in std_logic;
		master_hruser: in std_logic_vector(2 downto 0);
		reset_n: in std_logic
	);
end component;

component Gowin_CLKDIV
    port (
        clkout: out std_logic;
        hclkin: in std_logic;
        resetn: in std_logic
    );
end component;


signal master_exresp: std_logic;
signal master_hruser: std_logic_vector(2 downto 0);

signal data_mask: std_logic_vector (3 downto 0);
signal pll_lock: std_logic;

--HYPERRAM
signal W_HPRAM_WRDATA: std_logic_vector(31 downto 0);
signal hpram_data_out: std_logic_vector(31 downto 0);
signal rd_valid: std_logic;
signal W_reg_address: std_logic_vector(21 downto 0);
signal W_cross_reg_cmd2: std_logic;
signal W_cmd_enable: std_logic;
signal wire_init: std_logic;
signal user_clk: std_logic;

signal AHB_W_HPRAM_WRDATA: std_logic_vector(31 downto 0);
signal AHB_hpram_data_out: std_logic_vector(31 downto 0);
signal AHB_rd_valid: std_logic;
signal AHB_W_reg_address: std_logic_vector(21 downto 0);
signal AHB_W_cross_reg_cmd2: std_logic;
signal AHB_W_cmd_enable: std_logic;
signal AHB_wire_init: std_logic;
signal AHB_user_clk: std_logic;
signal AHB_data_mask: std_logic_vector(3 downto 0);

signal GB_wr_data           :   std_logic_vector(31 downto 0);
signal GB_rd_data           :   std_logic_vector(31 downto 0);
signal GB_rd_data_valid     :   std_logic;
signal GB_addr              :   std_logic_vector(21 downto 0);
signal GB_cmd               :   std_logic;
signal GB_cmd_en            :   std_logic;
signal GB_init_calib        :   std_logic;
signal GB_clk_out           :   std_logic;
signal GB_data_mask         :   std_logic_vector(3 downto 0);

signal GB_nAHB_MODE         : std_logic;

signal vcc: std_logic;
signal gnd: std_logic;


begin
    vcc <= '1';
    gnd <= '0';

 
    --CLOCK
    
    -- HYPERRAM
    u_Gowin_PLLVR: Gowin_PLLVR
    port map (
        clkout => hpram_memory_clk,
        clkoutd => hpram_base_clk,
        clkin => sys_clk
    );
    
    -- EMPU
--    CCLKDIV: Gowin_CLKDIV
--    port map (
--        clkout => mclk,     --6.75 MHz
--        hclkin => sys_clk,  --27 MHz
--        resetn => vcc
--    );
    
    
    --EMPU
    master_exresp <= '0';
    master_hruser <= "000";
    
    u_Gowin_EMPU_Top: Gowin_EMPU_Top
	port map (
		sys_clk => sys_clk,
		uart0_rxd => uart0_rxd,
		uart0_txd => uart0_txd,
		master_hclk => master_hclk,
		master_hrst => master_hrst,
		master_hsel => master_hsel,
		master_haddr => master_haddr,
		master_htrans => master_htrans,
		master_hwrite => master_hwrite,
		master_hsize => master_hsize,
		master_hburst => master_hburst,
		master_hprot => master_hprot,
		master_memattr => master_memattr,
		master_exreq => master_exreq,
		master_hmaster => master_hmaster,
		master_hwdata => master_hwdata,
		master_hmastlock => master_hmastlock,
		master_hreadymux => master_hreadymux,
		master_hauser => master_hauser,
		master_hwuser => master_hwuser,
		master_hrdata => master_hrdata,
		master_hreadyout => master_hreadyout,
		master_hresp => master_hresp,
		master_exresp => master_exresp,
		master_hruser => master_hruser,
		reset_n => vcc
	);
    
    --HYPERRAM
    pll_lock <= '1';
    
    u_HyperRAM_Memory_Interface_Top: HyperRAM_Memory_Interface_Top
	port map (
		clk             => hpram_base_clk,
		memory_clk      => hpram_memory_clk,
		pll_lock        => pll_lock,
		rst_n           => master_hrst,
		O_hpram_ck      => O_hpram_ck,
		O_hpram_ck_n    => O_hpram_ck_n,
		IO_hpram_dq     => IO_hpram_dq,
		IO_hpram_rwds   => IO_hpram_rwds,
		O_hpram_cs_n    => O_hpram_cs_n,
		O_hpram_reset_n => O_hpram_reset_n,
		wr_data         => W_HPRAM_WRDATA,
		rd_data         => hpram_data_out,
		rd_data_valid   => rd_valid,
		addr            => W_reg_address,
		cmd             => W_cross_reg_cmd2,
		cmd_en          => W_cmd_enable,
		init_calib      => wire_init,
		clk_out         => user_clk,
		data_mask       => data_mask
	);
    
    --AHB HYPERRAM BRIDGE
    u_Gowin_AHB_HyperRAM_Top: Gowin_AHB_HyperRAM_Top
    port map(
        AHB_HRDATA      =>  master_hrdata,    
        AHB_HREADY      =>  master_hreadyout,
        AHB_HRESP       =>  master_hresp,
        AHB_HTRANS      =>  master_htrans,
        AHB_HBURST      =>  master_hburst,
        AHB_HPROT       =>  master_hprot,
        AHB_HSIZE       =>  master_hsize,
        AHB_HWRITE      =>  master_hwrite,
        AHB_HMASTLOCK   =>  master_hmastlock,
        AHB_HMASTER     =>  master_hmaster,
        AHB_HADDR       =>  master_haddr,
        AHB_HWDATA      =>  master_hwdata,
        AHB_HSEL        =>  master_hsel,
        AHB_HCLK        =>  master_hclk,
        AHB_HRESETn     =>  master_hrst,

        --Gamboy Interface
        nWR             => nWR,
        nRD             => nRD,
        nCS             => nCS, 
        A               => A,
        A15             => A15,  
        D               => D,
        DIR             => DIR, 
        nOE             => nOE,  

        --Hyperram Interface
        W_HPRAM_WRDATA =>   W_HPRAM_WRDATA,
        hpram_data_out =>   hpram_data_out,
        rd_valid =>         rd_valid,
        W_reg_address =>    W_reg_address,
        W_cross_reg_cmd2 => W_cross_reg_cmd2,
        W_cmd_enable =>     W_cmd_enable,
        wire_init=>         wire_init,
        user_clk=>          user_clk,
        data_mask =>        data_mask,
        
        GB_RESET        =>  nRST,
        S_GB_nAHB_MODE    =>  GB_nAHB_MODE
      );
    
end rtl;



